@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2011 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -30,13 +28,11 @@
 
 /* draw_edit_form - draws an html edit form
    @arg $array - an array that contains all of the information needed to draw
-     the html form. see the arrays contained in include/config_settings.php
+     the html form. see the arrays contained in include/global_settings.php
      for the extact syntax of this array */
 function draw_edit_form($array) {
 	global $colors;
 
-	//print "<pre>";print_r($array);print "</pre>";
-
 	if (sizeof($array) > 0) {
 		while (list($top_branch, $top_children) = each($array)) {
 			if ($top_branch == "config") {
@@ -52,7 +48,7 @@ function draw_edit_form($array) {
 		while (list($field_name, $field_array) = each($fields_array)) {
 			if ($i == 0) {
 				if (!isset($config_array["no_form_tag"])) {
-					print "<form method='post' action='" . ((isset($config_array["post_to"])) ? $config_array["post_to"] : basename($_SERVER["PHP_SELF"])) . "'" . ((isset($config_array["form_name"])) ? " name='" . $config_array["form_name"] . "'" : "") . ">\n";
+					print "<tr style='display:none;'><td><form method='post' autocomplete='off' action='" . ((isset($config_array["post_to"])) ? $config_array["post_to"] : basename($_SERVER["PHP_SELF"])) . "'" . ((isset($config_array["form_name"])) ? " name='" . $config_array["form_name"] . "'" : "") . ((isset($config_array["enctype"])) ? " enctype='" . $config_array["enctype"] . "'" : "") . "></td></tr>\n";
 				}
 			}
 
@@ -61,18 +57,21 @@ function draw_edit_form($array) {
 			}elseif ($field_array["method"] == "hidden_zero") {
 				form_hidden_box($field_name, $field_array["value"], "0");
 			}elseif ($field_array["method"] == "spacer") {
-				print "<tr bgcolor='" . $colors["header_panel"] . "'><td colspan='2' class='textSubHeaderDark'>" . $field_array["friendly_name"] . "</td></tr>\n";
+				print "<tr id='row_$field_name' bgcolor='#" . $colors["header_panel"] . "'><td colspan='2' class='tableSubHeaderColumn'>" . $field_array["friendly_name"] . "</td></tr>\n";
 			}else{
 				if (isset($config_array["force_row_color"])) {
-					print "<tr bgcolor='#" . $config_array["force_row_color"] . "'>";
+					print "<tr id='row_$field_name' bgcolor='#" . $config_array["force_row_color"] . "'>";
 				}else{
-					form_alternate_row_color($colors["form_alternate1"],$colors["form_alternate2"],$i);
+					form_alternate_row_color($colors["form_alternate1"], $colors["form_alternate2"], $i, 'row_' . $field_name);
 				}
 
 				print "<td width='" . ((isset($config_array["left_column_width"])) ? $config_array["left_column_width"] : "50%") . "'>\n<font class='textEditTitle'>" . $field_array["friendly_name"] . "</font><br>\n";
 
 				if (isset($field_array["sub_checkbox"])) {
-					form_checkbox($field_array["sub_checkbox"]["name"], $field_array["sub_checkbox"]["value"], $field_array["sub_checkbox"]["friendly_name"], "", ((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+					form_checkbox($field_array["sub_checkbox"]["name"], $field_array["sub_checkbox"]["value"],
+							$field_array["sub_checkbox"]["friendly_name"], "",
+							((isset($check_array["on_change"])) ? $check_array["on_change"] : ""),
+							((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
 				}
 
 				print ((isset($field_array["description"])) ? $field_array["description"] : "") . "</td>\n";
@@ -82,94 +81,294 @@ function draw_edit_form($array) {
 				draw_edit_control($field_name, $field_array);
 
 				print "</td>\n</tr>\n";
-
-				$i++;
 			}
 
-			if ($i == sizeof($fields_array)) {
-				//print "</form>";
-			}
+			$i++;
 		}
 	}
 }
 
 /* draw_edit_control - draws a single control to be used on an html edit form
    @arg $field_name - the name of the control
-   @arg $field_array - an array containing data for this control. see include/config_form.php
+   @arg $field_array - an array containing data for this control. see include/global_form.php
      for more specific syntax */
 function draw_edit_control($field_name, &$field_array) {
 	switch ($field_array["method"]) {
 	case 'textbox':
-		form_text_box($field_name, $field_array["value"], ((isset($field_array["default"])) ? $field_array["default"] : ""), $field_array["max_length"], ((isset($field_array["size"])) ? $field_array["size"] : "40"), "text", ((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+		form_text_box($field_name, $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "text",
+			((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+
+		break;
+	case 'filepath':
+		form_filepath_box($field_name, $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "text",
+			((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+
+		break;
+	case 'dirpath':
+		form_dirpath_box($field_name, $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "text",
+			((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+
 		break;
 	case 'textbox_password':
-		form_text_box($field_name, $field_array["value"], ((isset($field_array["default"])) ? $field_array["default"] : ""), $field_array["max_length"], ((isset($field_array["size"])) ? $field_array["size"] : "40"), "password");
+		form_text_box($field_name, $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "password");
+
 		print "<br>";
-		form_text_box($field_name . "_confirm", $field_array["value"], ((isset($field_array["default"])) ? $field_array["default"] : ""), $field_array["max_length"], ((isset($field_array["size"])) ? $field_array["size"] : "40"), "password");
+
+		form_text_box($field_name . "_confirm", $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "password");
+
 		break;
 	case 'textarea':
-		form_text_area($field_name, $field_array["value"], $field_array["textarea_rows"], $field_array["textarea_cols"], ((isset($field_array["default"])) ? $field_array["default"] : ""));
+		form_text_area($field_name, $field_array["value"], $field_array["textarea_rows"],
+			$field_array["textarea_cols"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'drop_array':
-		form_dropdown($field_name, $field_array["array"], "", "", $field_array["value"], ((isset($field_array["none_value"])) ? $field_array["none_value"] : ""), ((isset($field_array["default"])) ? $field_array["default"] : ""));
+		form_dropdown($field_name, $field_array["array"], "", "", $field_array["value"],
+			((isset($field_array["none_value"])) ? $field_array["none_value"] : ""),
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'drop_sql':
-		form_dropdown($field_name, db_fetch_assoc($field_array["sql"]), "name", "id", $field_array["value"], ((isset($field_array["none_value"])) ? $field_array["none_value"] : ""), ((isset($field_array["default"])) ? $field_array["default"] : ""));
+		form_dropdown($field_name,
+			db_fetch_assoc($field_array["sql"]), "name", "id", $field_array["value"],
+				((isset($field_array["none_value"])) ? $field_array["none_value"] : ""),
+				((isset($field_array["default"])) ? $field_array["default"] : ""),
+				((isset($field_array["class"])) ? $field_array["class"] : ""),
+				((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'drop_multi':
-		form_multi_dropdown($field_name, $field_array["array"], db_fetch_assoc($field_array["sql"]), "id");
+		form_multi_dropdown($field_name, $field_array["array"], db_fetch_assoc($field_array["sql"]), "id",
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'drop_multi_rra':
-		form_multi_dropdown($field_name, array_rekey(db_fetch_assoc("select id,name from rra order by timespan"), "id", "name"), (empty($field_array["form_id"]) ? db_fetch_assoc($field_array["sql_all"]) : db_fetch_assoc($field_array["sql"])), "id");
+		form_multi_dropdown($field_name, array_rekey(db_fetch_assoc("select id,name from rra order by timespan"), "id", "name"),
+			(empty($field_array["form_id"]) ? db_fetch_assoc($field_array["sql_all"]) : db_fetch_assoc($field_array["sql"])), "id",
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'drop_tree':
 		grow_dropdown_tree($field_array["tree_id"], $field_name, $field_array["value"]);
+
 		break;
 	case 'drop_color':
-		form_color_dropdown($field_name, $field_array["value"], "None", ((isset($field_array["default"])) ? $field_array["default"] : ""));
+		form_color_dropdown($field_name, $field_array["value"], "None",
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'checkbox':
-		form_checkbox($field_name, $field_array["value"], $field_array["friendly_name"], ((isset($field_array["default"])) ? $field_array["default"] : ""), ((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+		form_checkbox($field_name,
+			$field_array["value"],
+			$field_array["friendly_name"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			((isset($field_array["form_id"])) ? $field_array["form_id"] : ""),
+			((isset($field_array["class"])) ? $field_array["class"] : ""),
+			((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 		break;
 	case 'checkbox_group':
 		while (list($check_name, $check_array) = each($field_array["items"])) {
-			form_checkbox($check_name, $check_array["value"], $check_array["friendly_name"], ((isset($check_array["default"])) ? $check_array["default"] : ""), ((isset($check_array["form_id"])) ? $check_array["form_id"] : ""));
+			form_checkbox($check_name, $check_array["value"], $check_array["friendly_name"],
+				((isset($check_array["default"])) ? $check_array["default"] : ""),
+				((isset($check_array["form_id"])) ? $check_array["form_id"] : ""),
+				((isset($field_array["class"])) ? $field_array["class"] : ""),
+				((isset($check_array["on_change"])) ? $check_array["on_change"] : (((isset($field_array["on_change"])) ? $field_array["on_change"] : ""))));
+
 			print "<br>";
 		}
+
 		break;
 	case 'radio':
 		while (list($radio_index, $radio_array) = each($field_array["items"])) {
-			form_radio_button($field_name, $field_array["value"], $radio_array["radio_value"], $radio_array["radio_caption"], ((isset($field_array["default"])) ? $field_array["default"] : ""));
+			form_radio_button($field_name, $field_array["value"], $radio_array["radio_value"], $radio_array["radio_caption"],
+				((isset($field_array["default"])) ? $field_array["default"] : ""),
+				((isset($field_array["class"])) ? $field_array["class"] : ""),
+				((isset($field_array["on_change"])) ? $field_array["on_change"] : ""));
+
 			print "<br>";
 		}
+
 		break;
 	case 'custom':
 		print $field_array["value"];
+
 		break;
 	case 'template_checkbox':
 		print "<em>" . html_boolean_friendly($field_array["value"]) . "</em>";
+
 		form_hidden_box($field_name, $field_array["value"], "");
+
 		break;
 	case 'template_drop_array':
 		print "<em>" . $field_array["array"]{$field_array["value"]} . "</em>";
+
 		form_hidden_box($field_name, $field_array["value"], "");
+
 		break;
 	case 'template_drop_multi_rra':
 		$items = db_fetch_assoc($field_array["sql_print"]);
 
 		if (sizeof($items) > 0) {
 		foreach ($items as $item) {
-			print $item["name"] . "<br>";
+			print htmlspecialchars($item["name"],ENT_QUOTES) . "<br>";
 		}
 		}
+
+		break;
+	case 'font':
+		form_font_box($field_name, $field_array["value"],
+			((isset($field_array["default"])) ? $field_array["default"] : ""),
+			$field_array["max_length"],
+			((isset($field_array["size"])) ? $field_array["size"] : "40"), "text",
+			((isset($field_array["form_id"])) ? $field_array["form_id"] : ""));
+
+		break;
+	case 'file':
+		form_file($field_name,
+			((isset($field_array["size"])) ? $field_array["size"] : "40"));
+
 		break;
 	default:
-		print "<em>" . $field_array["value"] . "</em>";
+		print "<em>" . htmlspecialchars($field_array["value"],ENT_QUOTES) . "</em>";
+
 		form_hidden_box($field_name, $field_array["value"], "");
+
 		break;
 	}
 }
 
+/* form_file - draws a standard html file input element
+   @arg $form_name - the name of this form element
+   @arg $form_size - the size (width) of the textbox */
+function form_file($form_name, $form_size = 30) {
+
+	print "<input type='file'";
+
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			print "class='txtErrorTextBox'";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
+	print " id='$form_name' name='$form_name' size='$form_size'>";
+}
+
+/* form_filepath_box - draws a standard html textbox and provides status of a files existence
+   @arg $form_name - the name of this form element
+   @arg $form_previous_value - the current value of this form element
+   @arg $form_default_value - the value of this form element to use if there is
+     no current value available
+   @arg $form_max_length - the maximum number of characters that can be entered
+     into this textbox
+   @arg $form_size - the size (width) of the textbox
+   @arg $type - the type of textbox, either 'text' or 'password'
+   @arg $current_id - used to determine if a current value for this form element
+     exists or not. a $current_id of '0' indicates that no current value exists,
+     a non-zero value indicates that a current value does exist */
+function form_filepath_box($form_name, $form_previous_value, $form_default_value, $form_max_length, $form_size = 30, $type = "text", $current_id = 0) {
+	if (($form_previous_value == "") && (empty($current_id))) {
+		$form_previous_value = $form_default_value;
+	}
+
+	print "<input type='$type'";
+
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			print "class='txtErrorTextBox'";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
+	if (isset($_SESSION["sess_field_values"])) {
+		if (!empty($_SESSION["sess_field_values"][$form_name])) {
+			$form_previous_value = $_SESSION["sess_field_values"][$form_name];
+		}
+	}
+
+	if (is_file(trim($form_previous_value))) {
+		$extra_data = "<span style='color:green'><br>[OK: FILE FOUND]</span>";
+	}else if (is_dir(trim($form_previous_value))) {
+		$extra_data = "<span style='color:red'><br>[ERROR: IS DIR]</span>";
+	}else if (strlen($form_previous_value) == 0) {
+		$extra_data = "";
+	}else{
+		$extra_data = "<span style='color:red'><br>[ERROR: FILE NOT FOUND]</span>";
+	}
+
+	print " id='$form_name' name='$form_name' size='$form_size'" . (!empty($form_max_length) ? " maxlength='$form_max_length'" : "") . " value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'>" . $extra_data;
+}
+
+/* form_dirpath_box - draws a standard html textbox and provides status of a directories existence
+   @arg $form_name - the name of this form element
+   @arg $form_previous_value - the current value of this form element
+   @arg $form_default_value - the value of this form element to use if there is
+     no current value available
+   @arg $form_max_length - the maximum number of characters that can be entered
+     into this textbox
+   @arg $form_size - the size (width) of the textbox
+   @arg $type - the type of textbox, either 'text' or 'password'
+   @arg $current_id - used to determine if a current value for this form element
+     exists or not. a $current_id of '0' indicates that no current value exists,
+     a non-zero value indicates that a current value does exist */
+function form_dirpath_box($form_name, $form_previous_value, $form_default_value, $form_max_length, $form_size = 30, $type = "text", $current_id = 0) {
+	if (($form_previous_value == "") && (empty($current_id))) {
+		$form_previous_value = $form_default_value;
+	}
+
+	print "<input type='$type'";
+
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			print "class='txtErrorTextBox'";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
+	if (isset($_SESSION["sess_field_values"])) {
+		if (!empty($_SESSION["sess_field_values"][$form_name])) {
+			$form_previous_value = $_SESSION["sess_field_values"][$form_name];
+		}
+	}
+
+	if (is_dir($form_previous_value)) {
+		$extra_data = "<span style='color:green'><br>[OK: DIR FOUND]";
+	}else if (is_file($form_previous_value)) {
+		$extra_data = "<span style='color:red'><br>[ERROR: IS FILE]";
+	}else if (strlen($form_previous_value) == 0) {
+		$extra_data = "";
+	}else{
+		$extra_data = "<span style='color:red'><br>[ERROR: DIR NOT FOUND]";
+	}
+
+	print " id='$form_name' name='$form_name' size='$form_size'" . (!empty($form_max_length) ? " maxlength='$form_max_length'" : "") . " value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'>" . $extra_data;
+}
+
 /* form_text_box - draws a standard html textbox
    @arg $form_name - the name of this form element
    @arg $form_previous_value - the current value of this form element
@@ -202,7 +401,7 @@ function form_text_box($form_name, $form_previous_value, $form_default_value, $f
 		}
 	}
 
-	print " name='$form_name' size='$form_size'" . (!empty($form_max_length) ? " maxlength='$form_max_length'" : "") . " value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'>\n";
+	print " id='$form_name' name='$form_name' size='$form_size'" . (!empty($form_max_length) ? " maxlength='$form_max_length'" : "") . " value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'>\n";
 }
 
 /* form_hidden_box - draws a standard html hidden element
@@ -215,7 +414,7 @@ function form_hidden_box($form_name, $form_previous_value, $form_default_value)
 		$form_previous_value = $form_default_value;
 	}
 
-	print "<input type='hidden' name='$form_name' value='$form_previous_value'>\n";
+	print "<div style='display:none;'><input type='hidden' id='$form_name' name='$form_name' value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'></div>\n";
 }
 
 /* form_dropdown - draws a standard html dropdown box
@@ -234,19 +433,35 @@ function form_hidden_box($form_name, $form_previous_value, $form_default_value)
    @arg $form_none_entry - the name to use for a default 'none' element in the dropdown
    @arg $form_default_value - the value of this form element to use if there is
      no current value available
-   @arg $css_style - any css that needs to be applied to this form element */
-function form_dropdown($form_name, $form_data, $column_display, $column_id, $form_previous_value, $form_none_entry, $form_default_value, $css_style = "") {
+   @arg $css_class - any css that needs to be applied to this form element
+   @arg $on_change - onChange modifier */
+function form_dropdown($form_name, $form_data, $column_display, $column_id, $form_previous_value, $form_none_entry, $form_default_value, $class = "", $on_change = "") {
 	if ($form_previous_value == "") {
 		$form_previous_value = $form_default_value;
 	}
 
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			$class .= (strlen($class) ? " ":"") . "txtErrorTextBox";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
 	if (isset($_SESSION["sess_field_values"])) {
 		if (!empty($_SESSION["sess_field_values"][$form_name])) {
 			$form_previous_value = $_SESSION["sess_field_values"][$form_name];
 		}
 	}
 
-	print "<select name='$form_name' style='$css_style'>";
+	if (strlen($class)) {
+		$class = " class='$class' ";
+	}
+
+	if (strlen($on_change)) {
+		$on_change = " onChange='$on_change' ";
+	}
+
+	print "<select id='" . htmlspecialchars($form_name) . "' name='" . htmlspecialchars($form_name) . "'" . $class . $on_change . ">";
 
 	if (!empty($form_none_entry)) {
 		print "<option value='0'" . (empty($form_previous_value) ? " selected" : "") . ">$form_none_entry</option>\n";
@@ -263,10 +478,11 @@ function form_dropdown($form_name, $form_data, $column_display, $column_id, $for
    @arg $form_caption - the text to display to the right of the checkbox
    @arg $form_default_value - the value of this form element to use if there is
      no current value available
+   @arg $on_change - specify a javascript onchange action
    @arg $current_id - used to determine if a current value for this form element
      exists or not. a $current_id of '0' indicates that no current value exists,
      a non-zero value indicates that a current value does exist */
-function form_checkbox($form_name, $form_previous_value, $form_caption, $form_default_value, $current_id = 0) {
+function form_checkbox($form_name, $form_previous_value, $form_caption, $form_default_value, $current_id = 0, $class = "", $on_change = "") {
 	if (($form_previous_value == "") && (empty($current_id))) {
 		$form_previous_value = $form_default_value;
 	}
@@ -277,7 +493,21 @@ function form_checkbox($form_name, $form_previous_value, $form_caption, $form_de
 		}
 	}
 
-	print "<input type='checkbox' name='$form_name' id='$form_name'" . (($form_previous_value == "on") ? " checked" : "") . "> $form_caption\n";
+	if (strlen($class)) {
+		$class = " class='$class'";
+	}
+
+	if (strlen($on_change)) {
+		$on_change = " onChange='$on_change'";
+	}
+
+	if ($form_previous_value == "on") {
+		$checked = " checked";
+	}else{
+		$checked = "";
+	}
+
+	print "<input type='checkbox' id='$form_name' name='$form_name'" . $on_change . $class . $checked . ">" . ($form_caption != "" ? " <label for='$form_name'>$form_caption</label>\n":"");
 }
 
 /* form_text_box - draws a standard html radio button
@@ -287,7 +517,7 @@ function form_checkbox($form_name, $form_previous_value, $form_caption, $form_de
    @arg $form_caption - the text to display to the right of the checkbox
    @arg $form_default_value - the value of this form element to use if there is
      no current value available */
-function form_radio_button($form_name, $form_previous_value, $form_current_value, $form_caption, $form_default_value) {
+function form_radio_button($form_name, $form_previous_value, $form_current_value, $form_caption, $form_default_value, $class = "", $on_change = "") {
 	if ($form_previous_value == "") {
 		$form_previous_value = $form_default_value;
 	}
@@ -298,7 +528,23 @@ function form_radio_button($form_name, $form_previous_value, $form_current_value
 		}
 	}
 
-	print "<input type='radio' name='$form_name' value='$form_current_value'" . (($form_previous_value == $form_current_value) ? " checked" : "") . "> $form_caption\n";
+	if (strlen($class)) {
+		$class = " class='$class' ";
+	}
+
+	if (strlen($on_change)) {
+		$on_change = " onChange='$on_change' ";
+	}
+
+	if ($form_previous_value == $form_current_value) {
+		$checked = " checked";
+	}else{
+		$checked = "";
+	}
+
+	$css_id = $form_name . "_" . $form_current_value;
+
+	print "<input type='radio' id='$css_id' name='$form_name' value='$form_current_value'" . $class . $on_change . $checked . "><label for='$css_id'>$form_caption</label>\n";
 }
 
 /* form_text_box - draws a standard html text area box
@@ -308,18 +554,33 @@ function form_radio_button($form_name, $form_previous_value, $form_current_value
    @arg $form_columns - the number of columns in the text area box
    @arg $form_default_value - the value of this form element to use if there is
      no current value available */
-function form_text_area($form_name, $form_previous_value, $form_rows, $form_columns, $form_default_value) {
+function form_text_area($form_name, $form_previous_value, $form_rows, $form_columns, $form_default_value, $class = "", $on_change = "") {
 	if ($form_previous_value == "") {
 		$form_previous_value = $form_default_value;
 	}
 
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			$class .= (strlen($class) ? " ":"") . "txtErrorTextBox";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
 	if (isset($_SESSION["sess_field_values"])) {
 		if (!empty($_SESSION["sess_field_values"][$form_name])) {
 			$form_previous_value = $_SESSION["sess_field_values"][$form_name];
 		}
 	}
 
-	print "<textarea cols='$form_columns' rows='$form_rows' name='$form_name'>" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "</textarea>\n";
+	if (strlen($class)) {
+		$class = " class='$class' ";
+	}
+
+	if (strlen($on_change)) {
+		$on_change = " onChange='$on_change' ";
+	}
+
+	print "<textarea cols='$form_columns' rows='$form_rows' id='$form_name' name='$form_name'" . $class . $on_change . ">" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "</textarea>\n";
 }
 
 /* form_multi_dropdown - draws a standard html multiple select dropdown
@@ -331,8 +592,24 @@ function form_text_area($form_name, $form_previous_value, $form_rows, $form_colu
      it must be formatted like:
      $array[0][$column_id] = key
    @arg $column_id - the name of the key used to reference the keys above */
-function form_multi_dropdown($form_name, $array_display, $sql_previous_values, $column_id) {
-	print "<select name='$form_name" . "[]' multiple>\n";
+function form_multi_dropdown($form_name, $array_display, $sql_previous_values, $column_id, $class = "", $on_change = "") {
+
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			$class .= (strlen($class) ? " ":"") . "txtErrorTextBox";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
+	if (strlen($class)) {
+		$class = " class='$class' ";
+	}
+
+	if (strlen($on_change)) {
+		$on_change = " onChange='$on_change' ";
+	}
+
+	print "<select id='$form_name' name='$form_name" . "[]'" . $class . " multiple>\n";
 
 	foreach (array_keys($array_display) as $id) {
 		print "<option value='" . $id . "'";
@@ -343,7 +620,7 @@ function form_multi_dropdown($form_name, $array_display, $sql_previous_values, $
 			}
 		}
 
-		print ">". $array_display[$id];
+		print ">". htmlspecialchars($array_display[$id],ENT_QUOTES);
 		print "</option>\n";
 	}
 
@@ -362,14 +639,26 @@ function form_multi_dropdown($form_name, $array_display, $sql_previous_values, $
    @arg $form_none_entry - the name to use for a default 'none' element in the dropdown
    @arg $form_default_value - the value of this form element to use if there is
      no current value available */
-function form_color_dropdown($form_name, $form_previous_value, $form_none_entry, $form_default_value) {
+function form_color_dropdown($form_name, $form_previous_value, $form_none_entry, $form_default_value, $class = "", $on_change = "") {
 	if ($form_previous_value == "") {
 		$form_previous_value = $form_default_value;
 	}
 
+	if (strlen($class)) {
+		$class = " class='$class' ";
+	}
+
+	$current_color = db_fetch_cell("SELECT hex FROM colors WHERE id=$form_previous_value");
+
+	if (strlen($on_change)) {
+		$on_change = " " . $on_change . ";";
+	}
+
+	$on_change = " onChange='this.style.backgroundColor=this.options[this.selectedIndex].style.backgroundColor;$on_change'";
+
 	$colors_list = db_fetch_assoc("select id,hex from colors order by hex desc");
 
-	print "<select name='$form_name'>\n";
+	print "<select style='background-color: #$current_color;' id='$form_name' name='$form_name'" . $class . $on_change . ">\n";
 
 	if ($form_none_entry != "") {
 		print "<option value='0'>$form_none_entry</option>\n";
@@ -377,7 +666,7 @@ function form_color_dropdown($form_name, $form_previous_value, $form_none_entry,
 
 	if (sizeof($colors_list) > 0) {
 		foreach ($colors_list as $color) {
-			print "<option style='background: #" . $color["hex"] . ";' value='" . $color["id"] . "'";
+			print "<option style='background-color: #" . $color["hex"] . ";' value='" . $color["id"] . "'";
 
 			if ($form_previous_value == $color["id"]) {
 				print " selected";
@@ -390,29 +679,87 @@ function form_color_dropdown($form_name, $form_previous_value, $form_none_entry,
 	print "</select>\n";
 }
 
+/* form_font_box - draws a standard html textbox and provides status of a fonts existence
+   @arg $form_name - the name of this form element
+   @arg $form_previous_value - the current value of this form element
+   @arg $form_default_value - the value of this form element to use if there is
+     no current value available
+   @arg $form_max_length - the maximum number of characters that can be entered
+     into this textbox
+   @arg $form_size - the size (width) of the textbox
+   @arg $type - the type of textbox, either 'text' or 'password'
+   @arg $current_id - used to determine if a current value for this form element
+     exists or not. a $current_id of '0' indicates that no current value exists,
+     a non-zero value indicates that a current value does exist */
+function form_font_box($form_name, $form_previous_value, $form_default_value, $form_max_length, $form_size = 30, $type = "text", $current_id = 0) {
+	global $config;
+
+	if (($form_previous_value == "") && (empty($current_id))) {
+		$form_previous_value = $form_default_value;
+	}
+
+	print "<input type='$type'";
+
+	if (isset($_SESSION["sess_error_fields"])) {
+		if (!empty($_SESSION["sess_error_fields"][$form_name])) {
+			print "class='txtErrorTextBox'";
+			unset($_SESSION["sess_error_fields"][$form_name]);
+		}
+	}
+
+	if (isset($_SESSION["sess_field_values"])) {
+		if (!empty($_SESSION["sess_field_values"][$form_name])) {
+			$form_previous_value = $_SESSION["sess_field_values"][$form_name];
+		}
+	}
+
+	if (strlen($form_previous_value) == 0) { # no data: defaults are used; everythings fine
+			$extra_data = "";
+	} else {
+
+		/* do some simple checks */
+		if (read_config_option("rrdtool_version") == "rrd-1.0.x" ||
+			read_config_option("rrdtool_version") == "rrd-1.2.x") { # rrdtool 1.0 and 1.2 use font files
+			if (is_file($form_previous_value)) {
+				$extra_data = "<span style='color:green'><br>[" . "OK: FILE FOUND" . "]</span>";
+			}else if (is_dir($form_previous_value)) {
+				$extra_data = "<span style='color:red'><br>[" . "ERROR: IS DIR" . "]</span>";
+			}else{
+				$extra_data = "<span style='color:red'><br>[" . "ERROR: FILE NOT FOUND" . "]</span>";
+			}
+		} else {	# rrdtool 1.3+ use fontconfig
+			/* verifying all possible pango font params is too complex to be tested here
+			 * so we only escape the font
+			 */
+			$extra_data = "<span style='color:green'><br>[" . "NO FONT VERIFICATION POSSIBLE" . "]</span>";
+		}
+	}
+
+	print " id='$form_name' name='$form_name' size='$form_size'" . (!empty($form_max_length) ? " maxlength='$form_max_length'" : "") . " value='" . htmlspecialchars($form_previous_value, ENT_QUOTES) . "'>" . $extra_data;
+}
+
 /* form_confirm - draws a table presenting the user with some choice and allowing
      them to either proceed (delete) or cancel
    @arg $body_text - the text to prompt the user with on this form
    @arg $cancel_url - the url to go to when the user clicks 'cancel'
    @arg $action_url - the url to go to when the user clicks 'delete' */
 function form_confirm($title_text, $body_text, $cancel_url, $action_url) { ?>
-		<br>
-		<table align="center" cellpadding=1 cellspacing=0 border=0 bgcolor="#B61D22" width="60%">
-			<tr>
-				<td bgcolor="#B61D22" colspan="10">
-					<table width="100%" cellpadding="3" cellspacing="0">
-						<tr>
-							<td bgcolor="#B61D22" class="textHeaderDark"><strong><?php print $title_text;?></strong></td>
-						</tr>
-						<?php
-						form_area($body_text);
-						form_confirm_buttons($action_url, $cancel_url);
-						?>
-					</table>
-				</td>
-			</tr>
-		</table>
-
+	<br>
+	<table align="center" cellpadding="1" cellspacing="0" border="0" bgcolor="#B61D22" width="60%">
+		<tr>
+			<td bgcolor="#B61D22" colspan="10">
+				<table width="100%" cellpadding="3" cellspacing="0">
+					<tr>
+						<td bgcolor="#B61D22" class="textHeaderDark"><strong><?php print $title_text;?></strong></td>
+					</tr>
+					<?php
+					form_area($body_text);
+					form_confirm_buttons($action_url, $cancel_url);
+					?>
+				</table>
+			</td>
+		</tr>
+	</table>
 <?php }
 
 /* form_confirm_buttons - draws a cancel and delete button suitable for display
@@ -422,8 +769,8 @@ function form_confirm($title_text, $body_text, $cancel_url, $action_url) { ?>
 function form_confirm_buttons($action_url, $cancel_url) { ?>
 	<tr>
 		<td bgcolor="#E1E1E1">
-			<a href="<?php print $cancel_url;?>"><img src="images/button_cancel.gif" border="0" alt="Cancel" align="absmiddle"></a>
-			<a href="<?php print $action_url . "&confirm=yes";?>"><img src="images/button_delete.gif" border="0" alt="Delete" align="absmiddle"></a>
+			<input type='button' onClick='cactiReturnTo("<?php print $cancel_url;?>")' value='Cancel'>
+			<input type='submit' onClick='cactiReturnTo("<?php print $action_url;?>&confirm=true")' value='Delete'>
 		</td>
 	</tr>
 <?php }
@@ -434,28 +781,44 @@ function form_confirm_buttons($action_url, $cancel_url) { ?>
    @arg $force_type - if specified, will force the 'action' button to be either
      'save' or 'create'. otherwise this field should be properly auto-detected */
 function form_save_button($cancel_url, $force_type = "", $key_field = "id") {
-	if (empty($force_type)) {
+	$calt = "Cancel";
+
+	if (empty($force_type) || $force_type == "return") {
 		if (empty($_GET[$key_field])) {
-			$img = "button_create.gif";
 			$alt = "Create";
 		}else{
-			$img = "button_save.gif";
 			$alt = "Save";
+
+			if (strlen($force_type)) {
+				$calt   = "Return";
+			}else{
+				$calt   = "Cancel";
+			}
 		}
+
 	}elseif ($force_type == "save") {
-		$img = "button_save.gif";
 		$alt = "Save";
 	}elseif ($force_type == "create") {
-		$img = "button_create.gif";
 		$alt = "Create";
+	}elseif ($force_type == "import") {
+		$alt = "Import";
+	}elseif ($force_type == "export") {
+		$alt = "Export";
 	}
+
+	if ($force_type != "import" && $force_type != "export" && $force_type != "save") {
+		$cancel_action = "<input type='button' onClick='cactiReturnTo(\"" . $cancel_url . "\")' value='" . $calt . "'>";
+	}else{
+		$cancel_action = "";
+	}
+
 	?>
-	<table align='center' width='98%' style='background-color: #ffffff; border: 1px solid #bbbbbb;'>
+	<table align='center' width='100%' style='background-color: #ffffff; border: 1px solid #bbbbbb;'>
 		<tr>
-			 <td bgcolor="#f5f5f5" align="right">
+			<td bgcolor="#f5f5f5" align="right">
 				<input type='hidden' name='action' value='save'>
-				<a href='<?php print $cancel_url;?>'><img src='images/button_cancel2.gif' alt='Cancel' align='absmiddle' border='0'></a>
-				<input type='image' src='images/<?php print $img;?>' alt='<?php print $alt;?>' align='absmiddle'>
+				<?php print $cancel_action;?>
+				<input type='submit' value='<?php print $alt;?>'>
 			</td>
 		</tr>
 	</table>
