@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2011 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -29,69 +27,179 @@
 	and the data template
    @arg $data_template_id - the id of the data template to push out values for */
 function push_out_data_source_custom_data($data_template_id) {
-	/* get data_input_id */
-	$data_template = db_fetch_row("select
-		id,
-		data_input_id
-		from data_template_data
-		where data_template_id=$data_template_id
-		and local_data_id=0");
+	
+	/* valid data template id? */
+	if (empty($data_template_id)) { return 0; }
+
+	/* get data_input_id from template */
+	$data_template_data = db_fetch_row("SELECT " .
+			"id, " .
+			"data_input_id " .
+			"FROM data_template_data " .
+			"WHERE data_template_id=$data_template_id " .
+			"AND local_data_id=0");
 
 	/* must be a data template */
-	if ((empty($data_template_id)) || (empty($data_template["data_input_id"]))) { return 0; }
+	if (empty($data_template_data["data_input_id"])) { return 0; }
 
 	/* get a list of data sources using this template */
-	$data_sources = db_fetch_assoc("select
-		data_template_data.id
-		from data_template_data
-		where data_template_id=$data_template_id
-		and local_data_id>0");
+	$data_sources = db_fetch_assoc("SELECT " .
+			"data_template_data.id " .
+			"FROM data_template_data " .
+			"WHERE data_template_id=$data_template_id " .
+			"AND local_data_id>0");
 
-	/* pull out all 'input' values so we know how much to save */
-	$input_fields = db_fetch_assoc("select
-		data_input_fields.id,
-		data_input_fields.type_code,
-		data_input_data.value,
-		data_input_data.t_value
-		from data_input_fields left join data_input_data
-		on (data_input_fields.id=data_input_data.data_input_field_id)
-		where data_input_data.data_template_data_id=" . $data_template["id"] . "
-		and data_input_fields.input_output='in'");
+	/* pull out all custom templated 'input' values from the template itself 
+	 * templated items are selected by querying t_value = '' OR t_value = NULL */
+	$template_input_fields = array_rekey(db_fetch_assoc("SELECT " .
+			"data_input_fields.id, " .
+			"data_input_fields.type_code, " .
+			"data_input_data.value, " .
+			"data_input_data.t_value " .
+			"FROM data_input_fields " .
+			"INNER JOIN (data_template_data " .
+			"INNER JOIN data_input_data " .
+			"ON data_template_data.id = data_input_data.data_template_data_id) " .
+			"ON data_input_fields.id = data_input_data.data_input_field_id " .
+			"WHERE (data_input_fields.input_output='in') " .
+			"AND (data_input_data.t_value='' OR data_input_data.t_value IS NULL) " .
+			"AND (data_input_data.data_template_data_id=" . $data_template_data["id"] . ") " .
+			"AND (data_template_data.local_data_template_data_id=0)"), "id", array("type_code", "value", "t_value"));
 
-	$data_rra = db_fetch_assoc("select rra_id from data_template_data_rra where data_template_data_id=" . $data_template["id"]);
+	/* which data_input_fields are templated? */
+	$dif_ct = 0;
+	$dif_in_str = "";
+	if (sizeof($template_input_fields)) {
+		$dif_in_str .= "AND data_input_fields.id IN (";
+		foreach ($template_input_fields as $key => $value) {
+			$dif_in_str .= ($dif_ct == 0 ? "":",") . $key;
+			$dif_ct++;
+		}
+		$dif_in_str .= ") ";
+	}
 
-	if (sizeof($data_sources) > 0) {
-	foreach ($data_sources as $data_source) {
-		reset($input_fields);
+	/* pull out all templated 'input' values from all related data sources 
+	 * unfortunately, you can't simply provide the same test as above
+	 * all input fields not related to a template ALWAYS are marked with t_value = NULL
+	 * so we will verify against the list of data_input_field id's taken from above */
+	$input_fields = db_fetch_assoc("SELECT " .
+			"data_template_data.id AS data_template_data_id, " .
+			"data_input_fields.id, " .
+			"data_input_fields.type_code, " .
+			"data_input_data.value, " .
+			"data_input_data.t_value " .
+			"FROM data_input_fields " .
+			"INNER JOIN (data_template_data " .
+			"INNER JOIN data_input_data " .
+			"ON data_template_data.id = data_input_data.data_template_data_id) " .
+			"ON data_input_fields.id = data_input_data.data_input_field_id " .
+			"WHERE (data_input_fields.input_output='in') " . 
+			$dif_in_str .
+			"AND (data_input_data.t_value='' OR data_input_data.t_value IS NULL) " .
+			"AND (data_template_data.local_data_template_data_id=" . $data_template_data["id"] . ")");
 
-		if (sizeof($input_fields) > 0) {
-		foreach ($input_fields as $input_field) {
-			/* do not push out "host fields" */
-			if (!eregi('^' . VALID_HOST_FIELDS . '$', $input_field["type_code"])) {
-				/* this is not a "host field", so we should either push out the value if it is templated
-				or leave it alone if the user checked "Use Per-Data Source Value". */
-				if ($input_field["t_value"] == "") { /* template this value */
-					db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,value) values (" . $input_field["id"] . "," . $data_source["id"] . ",'" . addslashes($input_field["value"]) . "')");
+	$data_rra = db_fetch_assoc("SELECT rra_id
+		FROM data_template_data_rra
+		WHERE data_template_data_id=" . $data_template_data["id"]);
+
+	/* perform bulk update of rra associations */
+	$rra_ct = 0;
+	$rra_in_str = "";
+	if (sizeof($data_rra)) {
+		foreach($data_rra as $rra) {
+			$rra_in_str .= ($rra_ct == 0 ? "(":",") . $rra["rra_id"];
+			$rra_ct++;
+		}
+		$rra_in_str .= ")";
+	}
+
+	$ds_cnt    = 0;
+	$did_cnt   = 0;
+	$rra_cnt   = 0;
+	$ds_in_str = "";
+	$did_vals  = "";
+	$rra_vals  = "";
+	if (sizeof($data_sources)) {
+		foreach ($data_sources as $data_source) {
+			reset($input_fields);
+			
+			if (sizeof($input_fields)) {
+				foreach ($input_fields as $input_field) {
+					if ($data_source["id"] == $input_field["data_template_data_id"]) {
+						/* do not push out "host fields" */
+						if (!preg_match('/^' . VALID_HOST_FIELDS . '$/i', $input_field["type_code"])) {
+							/* this is not a "host field", so we should either push out the value if it is templated */
+							$did_vals .= ($did_cnt == 0 ? "":",") . "(" . $input_field["id"] . ", " . $data_source["id"] . ", '" . addslashes($template_input_fields[$input_field["id"]]["value"]) . "')";
+							$did_cnt++;
+						}elseif ((isset($template_input_fields[$input_field["id"]])) &&
+							($template_input_fields[$input_field["id"]]["value"] != $input_field["value"])) { # templated input field deviates from currenmt data source, so update required
+							$did_vals .= ($did_cnt == 0 ? "":",") . "(" . $input_field["id"] . ", " . $data_source["id"] . ", '" . addslashes($template_input_fields[$input_field["id"]]["value"]) . "')";
+							$did_cnt++;
+						}
+					}
 				}
-			}elseif (($input_field["t_value"] == "") && ($input_field["value"] != "")) {
-				/* we only template a "host field" when the user types something in the field. this way the data
-				template always overides the host if the user chooses to do so */
-				db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,value) values (" . $input_field["id"] . "," . $data_source["id"] . ",'" . addslashes($input_field["value"]) . "')");
+			}
+			
+			/* create large inserts to reduce turns */
+			$ds_in_str .= ($ds_cnt == 0 ? "(":",") . $data_source["id"];
+			if (sizeof($data_rra)) {
+				foreach ($data_rra as $rra) {
+					$rra_vals .= ($rra_cnt == 0 ? "":",") . "(" . $data_source["id"] . "," . $rra["rra_id"] . ")";
+					$rra_cnt++;
+				}
+			}
+			$ds_cnt++;
+			
+			/* per 1000 data source, update rows */
+			if ($ds_cnt % 1000 == 0) {
+				$ds_in_str .= ")";
+				push_out_data_source_templates($did_vals, $ds_in_str, $rra_vals, $rra_in_str);
+				$ds_cnt    = 0;
+				$did_cnt   = 0;
+				$rra_cnt   = 0;
+				$ds_in_str = "";
+				$did_vals  = "";
+				$rra_vals  = "";
 			}
 		}
-		}
-
-		/* make sure to update the 'data_template_data_rra' table for each data source */
-		db_execute("delete from data_template_data_rra where data_template_data_id=" . $data_source["id"]);
-
-		reset($data_rra);
-
-		if (sizeof($data_rra) > 0) {
-		foreach ($data_rra as $rra) {
-			db_execute("insert into data_template_data_rra (data_template_data_id,rra_id) values (" . $data_source["id"] . "," . $rra["rra_id"] . ")");
-		}
-		}
 	}
+	
+	if ($ds_cnt > 0) {
+		$ds_in_str .= ")";
+		push_out_data_source_templates($did_vals, $ds_in_str, $rra_vals, $rra_in_str);
+	}
+}
+
+/* push out changed data template fields to related data sources
+ * @parm string $did_vals	- data input data fields
+ * @parm string $ds_in_str	- all data sources, formatted as SQL "IN" clause
+ * @parm string $rra_vals	- new set of rra associations
+ * @parm string $rra_in_str	- all rra associations, formatted as SQL "IN" clause
+ */
+function push_out_data_source_templates($did_vals, $ds_in_str, $rra_vals, $rra_in_str) {
+
+	/* update all templated input fields */
+	if ($did_vals != "") {
+		db_execute("INSERT INTO data_input_data
+			(data_input_field_id,data_template_data_id,value)
+			VALUES $did_vals
+			ON DUPLICATE KEY UPDATE value=VALUES(value)");
+	}
+
+	/* remove old RRA associations */
+	if ($ds_in_str != "" && $rra_in_str != "") {
+		db_execute("DELETE
+			FROM data_template_data_rra
+			WHERE data_template_data_id IN $ds_in_str
+			AND rra_id NOT IN $rra_in_str");
+	}
+
+	/* ... and add new ones */
+	if ($rra_vals != "") {
+			db_execute("INSERT INTO data_template_data_rra
+				(data_template_data_id,rra_id)
+			VALUES $rra_vals
+			ON DUPLICATE KEY UPDATE rra_id=VALUES(rra_id)");
 	}
 }
 
@@ -111,7 +219,7 @@ function push_out_data_source_item($data_template_rrd_id) {
 	reset($struct_data_source_item);
 	while (list($field_name, $field_array) = each($struct_data_source_item)) {
 		/* are we allowed to push out the column? */
-		if (((empty($data_template_rrd{"t_" . $field_name})) || (ereg("FORCE:", $field_name))) && ((isset($data_template_rrd{"t_" . $field_name})) && (isset($data_template_rrd[$field_name])))) {
+		if (((empty($data_template_rrd{"t_" . $field_name})) || (preg_match("/FORCE:/", $field_name))) && ((isset($data_template_rrd{"t_" . $field_name})) && (isset($data_template_rrd[$field_name])))) {
 			db_execute("update data_template_rrd set $field_name='" . addslashes($data_template_rrd[$field_name]) . "' where local_data_template_rrd_id=" . $data_template_rrd["id"]);
 		}
 	}
@@ -132,7 +240,7 @@ function push_out_data_source($data_template_data_id) {
 	reset($struct_data_source);
 	while (list($field_name, $field_array) = each($struct_data_source)) {
 		/* are we allowed to push out the column? */
-		if (((empty($data_template_data{"t_" . $field_name})) || (ereg("FORCE:", $field_name))) && ((isset($data_template_data{"t_" . $field_name})) && (isset($data_template_data[$field_name])))) {
+		if (((empty($data_template_data{"t_" . $field_name})) || (preg_match("/FORCE:/", $field_name))) && ((isset($data_template_data{"t_" . $field_name})) && (isset($data_template_data[$field_name])))) {
 			db_execute("update data_template_data set $field_name='" . addslashes($data_template_data[$field_name]) . "' where local_data_template_data_id=" . $data_template_data["id"]);
 
 			/* update the title cache */
@@ -152,22 +260,22 @@ function change_data_template($local_data_id, $data_template_id) {
 	global $struct_data_source, $struct_data_source_item;
 
 	/* always update tables to new data template (or no data template) */
-	db_execute("update data_local set data_template_id=$data_template_id where id=$local_data_id");
+	db_execute("UPDATE data_local SET data_template_id=$data_template_id WHERE id=$local_data_id");
 
 	/* get data about the template and the data source */
-	$data = db_fetch_row("select * from data_template_data where local_data_id=$local_data_id");
+	$data = db_fetch_row("SELECT * FROM data_template_data WHERE local_data_id=$local_data_id");
 	$template_data = (($data_template_id == "0") ? $data : db_fetch_row("select * from data_template_data where local_data_id=0 and data_template_id=$data_template_id"));
 
 	/* determine if we are here for the first time, or coming back */
 	if ((db_fetch_cell("select local_data_template_data_id from data_template_data where local_data_id=$local_data_id") == "0") ||
-	(db_fetch_cell("select local_data_template_data_id from data_template_data where local_data_id=$local_data_id") == "")) {
+		(db_fetch_cell("select local_data_template_data_id from data_template_data where local_data_id=$local_data_id") == "")) {
 		$new_save = true;
 	}else{
 		$new_save = false;
 	}
 
 	/* some basic field values that ALL data sources should have */
-	$save["id"] = $data["id"];
+	$save["id"] = (isset($data["id"]) ? $data["id"] : 0);
 	$save["local_data_template_data_id"] = $template_data["id"];
 	$save["local_data_id"] = $local_data_id;
 	$save["data_template_id"] = $data_template_id;
@@ -188,8 +296,8 @@ function change_data_template($local_data_id, $data_template_id) {
 	$save["data_source_path"] = $data["data_source_path"];
 
 	$data_template_data_id = sql_save($save, "data_template_data");
-	$data_rrds_list = db_fetch_assoc("select * from data_template_rrd where local_data_id=$local_data_id");
-	$template_rrds_list = (($data_template_id == "0") ? $data_rrds_list : db_fetch_assoc("select * from data_template_rrd where local_data_id=0 and data_template_id=$data_template_id"));
+	$data_rrds_list = db_fetch_assoc("SELECT * FROM data_template_rrd WHERE local_data_id=$local_data_id");
+	$template_rrds_list = (($data_template_id == "0") ? $data_rrds_list : db_fetch_assoc("SELECT * FROM data_template_rrd WHERE local_data_id=0 AND data_template_id=$data_template_id"));
 
 	if (sizeof($data_rrds_list) > 0) {
 		/* this data source already has "child" items */
@@ -216,7 +324,9 @@ function change_data_template($local_data_id, $data_template_id) {
 	}
 
 	/* make sure to copy down script data (data_input_data) as well */
-	$data_input_data = db_fetch_assoc("select data_input_field_id,t_value,value from data_input_data where data_template_data_id=" . $template_data["id"]);
+	$data_input_data = db_fetch_assoc("SELECT data_input_field_id, t_value, value
+		FROM data_input_data
+		WHERE data_template_data_id=" . $template_data["id"]);
 
 	/* this section is before most everthing else so we can determine if this is a new save, by checking
 	the status of the 'local_data_template_data_id' column */
@@ -224,18 +334,26 @@ function change_data_template($local_data_id, $data_template_id) {
 	foreach ($data_input_data as $item) {
 		/* always propagate on a new save, only propagate templated fields thereafter */
 		if (($new_save == true) || (empty($item["t_value"]))) {
-			db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,t_value,value) values (" . $item["data_input_field_id"] . ",$data_template_data_id,'" . $item["t_value"] . "','" . $item["value"] . "')");
+			db_execute("REPLACE INTO data_input_data
+				(data_input_field_id,data_template_data_id,t_value,value)
+				VALUES (" . $item["data_input_field_id"] . ", $data_template_data_id, '" . $item["t_value"] . "', '" . $item["value"] . "')");
 		}
 	}
 	}
 
 	/* make sure to update the 'data_template_data_rra' table for each data source */
-	$data_rra = db_fetch_assoc("select rra_id from data_template_data_rra where data_template_data_id=" . $template_data["id"]);
-	db_execute("delete from data_template_data_rra where data_template_data_id=$data_template_data_id");
+	$data_rra = db_fetch_assoc("SELECT rra_id
+		FROM data_template_data_rra
+		WHERE data_template_data_id=" . $template_data["id"]);
+
+	db_execute("DELETE FROM data_template_data_rra
+		WHERE data_template_data_id=$data_template_data_id");
 
 	if (sizeof($data_rra) > 0) {
 	foreach ($data_rra as $rra) {
-		db_execute("insert into data_template_data_rra (data_template_data_id,rra_id) values ($data_template_data_id," . $rra["rra_id"] . ")");
+		db_execute("INSERT INTO data_template_data_rra
+			(data_template_data_id,rra_id)
+			VALUES ($data_template_data_id," . $rra["rra_id"] . ")");
 	}
 	}
 }
@@ -392,7 +510,7 @@ function change_graph_template($local_graph_id, $graph_template_id, $intrusive)
 	}
 
 	/* some basic field values that ALL graphs should have */
-	$save["id"] = $graph_list["id"];
+	$save["id"] = (isset($graph_list["id"]) ? $graph_list["id"] : 0);
 	$save["local_graph_template_graph_id"] = $template_graph_list["id"];
 	$save["local_graph_id"] = $local_graph_id;
 	$save["graph_template_id"] = $graph_template_id;
@@ -567,6 +685,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 		/* suggested values for snmp query code */
 		$suggested_values = db_fetch_assoc("select text,field_name from snmp_query_graph_sv where snmp_query_graph_id=" . $snmp_query_array["snmp_query_graph_id"] . " order by sequence");
 
+		$suggested_values_graph = array();
 		if (sizeof($suggested_values) > 0) {
 		foreach ($suggested_values as $suggested_value) {
 			/* once we find a match; don't try to find more */
@@ -574,7 +693,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 				$subs_string = substitute_snmp_query_data($suggested_value["text"], $host_id, $snmp_query_array["snmp_query_id"], $snmp_query_array["snmp_index"], read_config_option("max_data_query_field_length"));
 				/* if there are no '|' characters, all of the substitutions were successful */
 				if (!strstr($subs_string, "|query")) {
-					db_execute("update graph_templates_graph set " . $suggested_value["field_name"] . "='" . addslashes($suggested_value["text"]) . "' where local_graph_id=" . $cache_array["local_graph_id"]);
+					db_execute("UPDATE graph_templates_graph SET " . $suggested_value["field_name"] . "='" . $suggested_value["text"] . "' WHERE local_graph_id=" . $cache_array["local_graph_id"]);
 
 					/* once we find a working value, stop */
 					$suggested_values_graph[$graph_template_id]{$suggested_value["field_name"]} = true;
@@ -634,6 +753,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 			/* suggested values for snmp query code */
 			$suggested_values = db_fetch_assoc("select text,field_name from snmp_query_graph_rrd_sv where snmp_query_graph_id=" . $snmp_query_array["snmp_query_graph_id"] . " and data_template_id=" . $data_template["id"] . " order by sequence");
 
+			$suggested_values_ds = array();
 			if (sizeof($suggested_values) > 0) {
 			foreach ($suggested_values as $suggested_value) {
 				/* once we find a match; don't try to find more */
@@ -643,7 +763,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 					/* if there are no '|' characters, all of the substitutions were successful */
 					if (!strstr($subs_string, "|query")) {
 						if (sizeof(db_fetch_row("show columns from data_template_data like '" . $suggested_value["field_name"] . "'"))) {
-							db_execute("update data_template_data set " . $suggested_value["field_name"] . "='" . addslashes($suggested_value["text"]) . "' where local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
+							db_execute("UPDATE data_template_data SET " . $suggested_value["field_name"] . "='" . $suggested_value["text"] . "' WHERE local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
 						}
 
 						/* once we find a working value, stop */
@@ -651,7 +771,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 
 						if ((sizeof(db_fetch_row("show columns from data_template_rrd like '" . $suggested_value["field_name"] . "'"))) &&
 							(!substr_count($subs_string, "|"))) {
-							db_execute("update data_template_rrd set " . $suggested_value["field_name"] . "='" . $subs_string . "' where local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
+							db_execute("UPDATE data_template_rrd SET " . $suggested_value["field_name"] . "='" . $suggested_value["text"] . "' WHERE local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
 						}
 					}
 				}
@@ -660,25 +780,38 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 		}
 
 		if (is_array($snmp_query_array)) {
-			$data_input_field = array_rekey(db_fetch_assoc("select
+			$data_input_field = array_rekey(db_fetch_assoc("SELECT
 				data_input_fields.id,
 				data_input_fields.type_code
-				from (snmp_query,data_input,data_input_fields)
-				where snmp_query.data_input_id=data_input.id
-				and data_input.id=data_input_fields.data_input_id
-				and (data_input_fields.type_code='index_type' or data_input_fields.type_code='index_value' or data_input_fields.type_code='output_type')
-				and snmp_query.id=" . $snmp_query_array["snmp_query_id"]), "type_code", "id");
+				FROM (snmp_query,data_input,data_input_fields)
+				WHERE snmp_query.data_input_id=data_input.id
+				AND data_input.id=data_input_fields.data_input_id
+				AND (data_input_fields.type_code='index_type'
+					OR data_input_fields.type_code='index_value'
+					OR data_input_fields.type_code='output_type')
+				AND snmp_query.id=" . $snmp_query_array["snmp_query_id"]), "type_code", "id");
 
-			$snmp_cache_value = db_fetch_cell("select field_value from host_snmp_cache where host_id='$host_id' and snmp_query_id='" . $snmp_query_array["snmp_query_id"] . "' and field_name='" . $snmp_query_array["snmp_index_on"] . "' and snmp_index='" . $snmp_query_array["snmp_index"] . "'");
+			$snmp_cache_value = db_fetch_cell("SELECT field_value
+				FROM host_snmp_cache
+				WHERE host_id='$host_id'
+				AND snmp_query_id='" . $snmp_query_array["snmp_query_id"] . "'
+				AND field_name='" . $snmp_query_array["snmp_index_on"] . "'
+				AND snmp_index='" . $snmp_query_array["snmp_index"] . "'");
 
 			/* save the value to index on (ie. ifindex, ifip, etc) */
-			db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,t_value,value) values (" . $data_input_field["index_type"] . ",$data_template_data_id,'','" . $snmp_query_array["snmp_index_on"] . "')");
+			db_execute("REPLACE INTO data_input_data
+				(data_input_field_id, data_template_data_id, t_value, value)
+				VALUES (" . $data_input_field["index_type"] . ", $data_template_data_id, '', '" . $snmp_query_array["snmp_index_on"] . "')");
 
 			/* save the actual value (ie. 3, 192.168.1.101, etc) */
-			db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,t_value,value) values (" . $data_input_field["index_value"] . ",$data_template_data_id,'','" . addslashes($snmp_cache_value) . "')");
+			db_execute("REPLACE INTO data_input_data
+				(data_input_field_id,data_template_data_id,t_value,value)
+				VALUES (" . $data_input_field["index_value"] . ",$data_template_data_id,'','" . addslashes($snmp_cache_value) . "')");
 
 			/* set the expected output type (ie. bytes, errors, packets) */
-			db_execute("replace into data_input_data (data_input_field_id,data_template_data_id,t_value,value) values (" . $data_input_field["output_type"] . ",$data_template_data_id,'','" . $snmp_query_array["snmp_query_graph_id"] . "')");
+			db_execute("REPLACE INTO data_input_data
+				(data_input_field_id,data_template_data_id,t_value,value)
+				VALUES (" . $data_input_field["output_type"] . ",$data_template_data_id,'','" . $snmp_query_array["snmp_query_graph_id"] . "')");
 
 			/* now that we have put data into the 'data_input_data' table, update the snmp cache for ds's */
 			update_data_source_data_query_cache($cache_array["local_data_id"]{$data_template["id"]});
@@ -688,7 +821,9 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 		if (isset($suggested_values_array[$graph_template_id]["data_template"]{$data_template["id"]})) {
 			reset($suggested_values_array[$graph_template_id]["data_template"]{$data_template["id"]});
 			while (list($field_name, $field_value) = each($suggested_values_array[$graph_template_id]["data_template"]{$data_template["id"]})) {
-				db_execute("update data_template_data set $field_name='$field_value' where local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
+				db_execute("UPDATE data_template_data
+					SET $field_name='$field_value'
+					WHERE local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
 			}
 		}
 
@@ -698,7 +833,9 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 			while (list($data_template_item_id, $field_array) = each($suggested_values_array[$graph_template_id]["data_template_item"])) {
 				while (list($field_name, $field_value) = each($field_array)) {
 					$data_source_item_id = db_fetch_cell("select id from data_template_rrd where local_data_template_rrd_id=$data_template_item_id and local_data_id=" . $cache_array["local_data_id"]{$data_template["id"]});
-					db_execute("update data_template_rrd set $field_name='$field_value' where id=$data_source_item_id");
+					db_execute("UPDATE data_template_rrd
+						SET $field_name='$field_value'
+						WHERE id=$data_source_item_id");
 				}
 			}
 		}
